# Concept: recursion

Recursion is a programming technique where a function calls itself directly or indirectly in order to solve a problem. The idea behind recursion is to break down a complex problem into smaller, more manageable subproblems, each of which is a simpler instance of the original problem. Recursion is particularly useful for tasks that can be defined in terms of similar subtasks, such as tree traversal, factorial computation, and solving puzzles like the Tower of Hanoi.

## Key Characteristics of Recursion

- Base Case: The base case is a condition that stops the recursion. It is the simplest instance of the problem that can be solved directly, without further recursion.
- Recursive Case: The recursive case involves breaking the problem down into smaller instances of the same problem, with the function calling itself to solve these subproblems.
- Stack Usage: Each recursive call adds a new frame to the call stack, and when the base case is reached, the stack begins to unwind, resolving each recursive call.
- Potential Risks: Recursion can lead to issues such as stack overflow if the base case is not reached or if the recursion is too deep. Careful design is required to avoid these problems.

## Example 1: Factorial Calculation Using Recursion

One of the classic examples of recursion is the calculation of the factorial of a number.

```cpp
#include <iostream>

// Function to calculate factorial of n using recursion
int factorial(int n) {
    if (n == 0 || n == 1) {  // Base case
        return 1;
    } else {  // Recursive case
        return n * factorial(n - 1);
    }
}

int main() {
    int number = 5;
    std::cout << "Factorial of " << number << " is " << factorial(number) << std::endl;
    return 0;
}
```

### Explanation:
- Base Case: `if (n == 0 || n == 1) return 1;` - The factorial of `0` or `1` is `1`. This stops the recursion.
- Recursive Case: `return n * factorial(n - 1);` - The function calls itself with the value `n - 1`, reducing the problem size by one.
- Stack Unwinding: When the base case is reached, the function returns and each call in the stack multiplies the current value by the result of the next call.

## Example 2: Fibonacci Sequence Using Recursion
The Fibonacci sequence is another classic example where recursion can be used. The sequence is defined as:

```cpp
#include <iostream>

// Function to calculate factorial of n using recursion
int factorial(int n) {
    if (n == 0 || n == 1) {  // Base case
        return 1;
    } else {  // Recursive case
        return n * factorial(n - 1);
    }
}

int main() {
    int number = 5;
    std::cout << "Factorial of " << number << " is " << factorial(number) << std::endl;
    return 0;
}
```

### Explanation:

- Base Cases: `fibonacci(0)` returns `0` and `fibonacci(1)` returns `1`.
- Recursive Case: `return fibonacci(n - 1) + fibonacci(n - 2);` - The function calls itself twice, breaking the problem down into smaller Fibonacci calculations.
- Exponential Complexity: This naive implementation has exponential time complexity, leading to inefficiency for large `n`. Memoization or iterative approaches are often used to optimize it.

## Example 3: Recursive Tree Traversal

Recursion is particularly well-suited for tree structures, where each node might have its own subtrees. Here is an example of a simple binary tree traversal.

```cpp
#include <iostream>

struct Node {
    int data;
    Node* left;
    Node* right;
    Node(int val) : data(val), left(nullptr), right(nullptr) {}
};

// Function to perform an in-order traversal of the binary tree
void inOrderTraversal(Node* root) {
    if (root == nullptr) {
        return;  // Base case: if the node is null, return
    }

    inOrderTraversal(root->left);  // Traverse the left subtree
    std::cout << root->data << " ";  // Visit the root
    inOrderTraversal(root->right);  // Traverse the right subtree
}

int main() {
    // Create a simple binary tree
    Node* root = new Node(1);
    root->left = new Node(2);
    root->right = new Node(3);
    root->left->left = new Node(4);
    root->left->right = new Node(5);

    std::cout << "In-order traversal: ";
    inOrderTraversal(root);
    std::cout << std::endl;

    // Free the allocated memory (not shown for simplicity)
    return 0;
}
```

### Explanation:

- Base Case: `if (root == nullptr) return;` - If the node is null, the recursion stops.
- Recursive Case: The function recursively traverses the left subtree, visits the root, and then traverses the right subtree.
- Tree Traversal: This example demonstrates in-order traversal, but similar recursive approaches can be used for pre-order and post-order traversals.

## Example 4: Tower of Hanoi Problem
The Tower of Hanoi is a classic problem that is often solved using recursion. The goal is to move a set of disks from one rod to another, following specific rules.

```cpp
#include <iostream>

// Function to solve the Tower of Hanoi problem
void towerOfHanoi(int n, char source, char destination, char auxiliary) {
    if (n == 1) {
        std::cout << "Move disk 1 from " << source << " to " << destination << std::endl;
        return;  // Base case: Only one disk to move
    }

    towerOfHanoi(n - 1, source, auxiliary, destination);  // Move n-1 disks from source to auxiliary
    std::cout << "Move disk " << n << " from " << source << " to " << destination << std::endl;
    towerOfHanoi(n - 1, auxiliary, destination, source);  // Move n-1 disks from auxiliary to destination
}

int main() {
    int n = 3;  // Number of disks
    towerOfHanoi(n, 'A', 'C', 'B');  // A is source, C is destination, B is auxiliary
    return 0;
}
```

### Explanation:

- Base Case: `if (n == 1)` - If there is only one disk, move it directly to the destination.
- Recursive Case: The function moves `n-1` disks from the source to the auxiliary rod, then moves the nth disk to the destination, and finally moves the `n-1` disks from the auxiliary rod to the destination.
- Recursive Complexity: The Tower of Hanoi problem is solved in `2^n - 1` moves, where `n` is the number of disks.

## Recursion vs. Iteration

While recursion is a powerful tool, it's not always the best choice. In some cases, iterative solutions may be more efficient or easier to understand. For instance, deep recursion can lead to stack overflow if the recursion depth is too great, and some problems that are naturally recursive may have more efficient iterative solutions.

## Summary

- Recursion: A technique where a function calls itself to solve a problem. It breaks down a problem into smaller instances of itself.
- Base Case: The simplest case, which stops the recursion.
- Recursive Case: The case where the function calls itself with a reduced problem size.
- Applications: Recursion is commonly used in algorithms involving tree traversal, mathematical sequences (e.g., factorial, Fibonacci), puzzles (e.g., Tower of Hanoi), and more.
- Efficiency: Recursion can be elegant and easy to implement, but it may lead to inefficiencies and stack overflow in some cases. It’s important to consider iterative alternatives when appropriate.

Understanding recursion is fundamental to mastering algorithms and problem-solving in programming. It's a powerful tool that, when used appropriately, can simplify the implementation of complex problems.
